#################################################################################
# Created       : Martin Foster
# Modified      : 02/04/2001
#################################################################################
#
# Login - Modules part of Webchat designed to simplify login  
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2R 0K9
#
#################################################################################

package Webchat::Login;						# Package name

use Webchat::Table;						# Table Creation

#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Authenthication
#
# This is the primary component of this module, and this would be the 
# authenthication script, which will verify usernames and passwords.

sub Authenthication						# Database, Parameter hash, Choice
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# SQL statement handle 

	my $param = shift;					# Parameter list
	my $choice = shift;					# Choice of matter
	my $crypt;						# Crypted password
	my $check;						# Check registration	

	my %login;						# Login dbm


	#####################
	# Program area

	# Prepare and execute
	if ($choice == 1)					# Adminstrative
	{ 
		$Statement = $Database->prepare("SELECT PuppeteerPassword
					FROM Puppeteer 
					WHERE PuppeteerLogin=?
					  AND PuppeteerAdminPrivs='yes'"); 
	}

	if ($choice == 2)					# Supervisory
	{ 
		$Statement = $Database->prepare("SELECT PuppeteerPassword 
					FROM Puppeteer 
					WHERE PuppeteerLogin=?
					  AND PuppeteerSuperPrivs='yes'");
	}

	elsif ($choice == 99) 					# Non-Authoritative
	{ 
		$Statement = $Database->prepare("SELECT PuppeteerPassword 
					FROM Puppeteer 
					WHERE PuppeteerLogin=?"); 
	}
	$Statement->execute($param->{'USER'});

	# Retreive and finish
	$crypt = $Statement->fetchrow();
	$Statement->finish();

	# Compare passwords
	if (defined($crypt))
	{
		# If crypt
		if (defined($param->{'CRYPT'}))
		{
			# Fail on non-match
			if ($param->{'CRYPT'} ne $crypt)
			{ return 0; }
		}

		# If first time
		elsif (defined($param->{'PASS'}))
		{
			# Fail on non-match
			if (crypt($param->{'PASS'}, $crypt) ne $crypt)
			{ return 0; }

			# Otherwise assign crypt
			else
			{ $param->{'CRYPT'} = (crypt($param->{'PASS'}, $crypt)); }
		}

		# Assume fault
		else
		{ return 0; }
	}

	# Assume fault
	else
	{ return 0; }


	# Verify for incomplete registration
	# Prepare and execute
	$Statement = $Database->prepare("SELECT PuppeteerLogin 
			FROM PuppeteerRegistration
			WHERE PuppeteerLogin=?"); 
	$Statement->execute($param->{'USER'});

	# Retreive and finish
	$check = $Statement->fetchrow();
	$Statement->finish();
	
	# Check if registration is complete
	if (defined($check))
	{ return 0; }						# Or return false


	# Database updates
	# If non-authoritative
	if (defined($param->{'JAVA'}))
	{
		#####################
		# Data members
		my $radiojava;					# Formated javascript
		my $radiochat;					# Formatted chat preferences
		my $radioformat;				# Formatted format options
	

		# Link with hash
		Webchat::Database::GetHashLogin($Database, \%login);

		# Verify options received
		$radiojava = OptionFindJavascript($login{'OptJavascript'}, $param->{'JAVA'});
		$radiochat = OptionFindChatprefs($login{'OptChatPref'}, $param->{'CHAT'});
		$radioformat = OptionFindFormatting($login{'OptFormat'}, $param->{'FORMAT'});

		# Javascript options
		if ($radiojava ne 'default')
		{
			# Update database entry
			$Database->do("UPDATE Puppeteer SET PuppeteerJavascript=? WHERE PuppeteerLogin=?", 
					{}, $radiojava, $param->{'USER'});
		}
	
		# Chatting options
		if ($radiochat ne 'default')
		{
			# Update database entry
			$Database->do("UPDATE Puppeteer SET PuppeteerChatPref=? WHERE PuppeteerLogin=?", 
					{}, $radiochat, $param->{'USER'});
		}

		# Formatting options
		if ($radioformat ne 'default')
		{
			# Update database entry
			$Database->do("UPDATE Puppeteer SET PuppeteerFormatting=? WHERE PuppeteerLogin=?", 
					{}, $radioformat, $param->{'USER'});
		}
	}

	# Update database entry update timestamp
	$Database->do("UPDATE Puppeteer SET PuppeteerTimestamp=? WHERE PuppeteerLogin=?", 
			{}, time, $param->{'USER'});
	

	# Cleanup
	delete $param->{'JAVA'};
	delete $param->{'CHAT'};
	delete $param->{'FORMAT'};
	delete $param->{'PASS'};
	delete $param->{$login{'TxtSubmitButton'}};

	return 1;						# Return true
}

#####################
# GatherInfo
#
# Sub-routine that will handle the display of information, comes in two flavors
# adminstrative and non-authoritative login.   Selected though alias.

sub GatherInfo							# CGI, Parameter hash, Choice
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI Handle
	my $param = shift;					# Parameter list

	my $choice = shift;					# Choice on the matter

	my %login;						# Login hash


	#####################
	# Program area

	# Link with hash
	Webchat::Database::GetHashLogin($Database, \%login);


	# Print titles
	if ($choice == 99)					# Non-authoritative
	{ print "$login{'TagTitleGeneral'}\n<BR>\n"; }
 	elsif ($choice == 1)					# Adminstrative
	{ print "$login{'TagTitleAdmin'}\n<BR>\n"; }
	elsif ($choice == 2)
	{ print "$login{'TagTitleSuper'}\n<BR>\n"; }

	print $cgi->start_form();				# Start form
	Webchat::Table::MakeTop($cgi);				# Top of table

	# Username and password
	unless (defined($param->{'AUTH'}))
	{
		Webchat::Table::MakeValid($cgi, $login{'TagUsername'}, $cgi->textfield('USER', '', 15, 10));
		Webchat::Table::MakeValid($cgi, $login{'TagPassword'}, $cgi->password_field('PASS', '', 15, 8));
	}
	else
	{
		Webchat::Table::MakeInvalid($cgi, $login{'TagUsername'}, $cgi->textfield('USER', '', 15, 10));
		Webchat::Table::MakeInvalid($cgi, $login{'TagPassword'}, $cgi->password_field('PASS', '', 15, 8));
	}
	Webchat::Table::MakeBlank($cgi);			# Spacer


	# If non admistrative
	unless ($choice != 99)
	{
		#####################
		# Data members	
		my $radiojava;					# Formated javascript
		my $radiochat;					# Formatted chat preferences
		my $radioformat;				# Formatted format options

		my @radiojava;					# Javascript options
		my @radiochat;					# Chat options
		my @radioformat;				# Formatting options


		# Split data into arrays
		OptionSplit(\@radiojava, $login{'OptJavascript'});
		OptionSplit(\@radiochat, $login{'OptChatPref'});
		OptionSplit(\@radioformat, $login{'OptFormat'});

		# Format options nicely
		$radiojava   = $cgi->radio_group('JAVA',   \@radiojava,   $radiojava[2],   'true');
		$radiochat   = $cgi->radio_group('CHAT',   \@radiochat,   $radiochat[3],   'true');
		$radioformat = $cgi->radio_group('FORMAT', \@radioformat, $radioformat[3], 'true');

		# Javascript
		Webchat::Table::MakeValid($cgi, $login{'TagOptJavacript'}, $radiojava);
		Webchat::Table::MakeBlank($cgi);		# Spacer

		# Chat preferences
		Webchat::Table::MakeValid($cgi, $login{'TagOptChatPref'}, $radiochat);
		Webchat::Table::MakeBlank($cgi);		# Spacer

		# Format options
		Webchat::Table::MakeValid($cgi, $login{'TagOptFormat'}, $radioformat);
		Webchat::Table::MakeBlank($cgi);		# Spacer
	}

	# Submit and table bottom
	Webchat::Table::MakeValid($cgi, $cgi->submit($login{'TxtSubmitButton'}), $cgi->reset($login{'TxtResetButton'}));
	Webchat::Table::MakeBottom($cgi);

	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'AUTH', 'True');
	
	print $cgi->end_form();					# End of form
}



#####################
# Get Verification
#
# Primary entrance which will lead to GatherInfo or Authenthication depending
# on received parameters.

sub GetVerification						# Database, CGI, Parameter, Choice
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter reference
	my $choice = shift;					# Decision to make


	#####################
	# Program area
	
	# Verify is necessary components are there
	if ((defined($param->{'USER'})) && ((defined($param->{'PASS'})) || (defined($param->{'CRYPT'}))))
	{
		if (Authenthication($Database, $param, $choice))
		{ return 1; }					# Return true
	}
	
	GatherInfo($Database, $cgi, $param, $choice); 		# Reshow login
	return 0;						# Return false
}

# Alias for administrative
sub GetVerificationAdmin
{ GetVerification(shift, shift, shift, 1); }

# Alias for normal
sub GetVerificationNormal
{ GetVerification(shift, shift, shift, 99); }

# Alias for supervisory
sub GetVerificationSuper
{ GetVerification(shift, shift, shift, 2); }


#####################
# OptionSplit
#
# Splits options list into seperate elements of array

sub OptionSplit
{
	#####################
	# Data members

	my $array = shift;					# Reference to array
	my $data = shift;					# Retreived line


	#####################
	# Program area
	@{ $array } = split(/:/, $data);			# Split line
}


#####################
# OptionFind
#
# Splits options list into seperate elements of array and determines which
# option was specifically selected for the database

sub OptionFind
{
        #####################
        # Data members
        my $string = shift;                                     # String with all options
        my $val = shift;                                        # Value to search for
        my $choice = shift;                                     # Flow of events

        my $counter = 0;                                        # Basic counter

        my @values;                                             # Array of values


        #####################
        # Program area

        OptionSplit(\@values, $string);                         # Split options\receive values

        # Loop though array
        while ($string = shift @values)
        {
                # Verify for match
                if ($string eq $val)
                { last; }                                       # If found exit loop

                $counter++;                                     # Increment if no match found
        }

        # Javascript Options/Yes no Options
        if ($choice == 0)
        {
                # Which option applies
                if ($counter == 0) { return 'yes'; }
                elsif ($counter == 1) { return 'no'; }
        }

        # Chat preferences
        elsif ($choice == 1)
        {
                # Which option applies
                if ($counter == 0) { return 'basic'; }
                elsif ($counter == 1) { return 'framed'; }
                elsif ($counter == 2) { return 'stream'; }
        }

        # Format used in chat
        elsif ($choice == 2)
        {
                # Which option applies
                if ($counter == 0) { return 'complete'; }
                elsif ($counter == 1) { return 'imageless'; }
                elsif ($counter == 2) { return 'spartan'; }
        }

	return 'default';
}

# Alias for Javascript
sub OptionFindJavascript
{ return OptionFind(shift, shift, 0); }

# Alias for Chat preferences
sub OptionFindChatprefs
{ return OptionFind(shift, shift, 1); }

# Alias for Formatting
sub OptionFindFormatting
{ return OptionFind(shift, shift, 2); }

1;								# Return true
