#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 03/02/2001
#################################################################################
#
# Configure Puppet - Script part of Webchat designed to configure Puppets
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error Logs
#use strict;							# Strict variable enforcement

use Image::Grab;						# Image grabber
use Image::Size;						# Image sizer
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm

use Webchat::Database;						# Database handler
use Webchat::Filter;						# Filter system
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash
my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashConfiguration($Database, \%config);
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	# Authenthication
	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		print "$config{'TagTitle'}\n<BR>\n";		# Print title
	
		# For additions and removal
		unless  ((defined($param{$config{'TxtSubmitView'}})) || (defined($param{$config{'TxtSubmitButton'}})))
		{ InitialDisplay($Database, $cgi, \%param, \%config); }

		# For puppet changes and viewing
		else
		{ PuppetControl($Database, $cgi, \%param, \%config); }
	}

	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle


#################################################################################
# Sub-Routines
#################################################################################

#####################
# ImageSize
#
# Downloads a picture off the net and determins it's size as to be able place 
# the defined sizes on the database

sub ImageSize
{
        ####################
        # Data Members

	my $Database = shift;					# Database handle	
        my $link     = shift;					# URL to picture

	my $image;						# Image handle
        my $name;						# Name of file

	my %system;						# System hash
	my @sizes;						# Size returned


        ####################
        # Program Area

	# Link to Hash
	Webchat::Database::GetHashSystem($Database, \%system);

        # Initialize handles
        $image = new Image::Grab;

        # Generate random name
        $name = md5_hex(rand());;
	$name = "$system{'LocTemp'}/$name";

        # Retreive image
        $image->url($link);
        $image->grab;

        # Write to file
        open(IMAGE, ">$name") || die("Cannot open $name : $!");	# Open filehandle
        print IMAGE $image->image;				# Write to filehandle
        close(IMAGE);						# Close filehandle

	# Determine image size and remove file
	@sizes = imgsize("$name");
	unlink("$name");

	return @sizes;						# Return values
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my @puppets;						# Array of puppets

	my $list;						# List of puppets
	my $buttonview;						# Button view
	my $buttonrem;						# Button remove
	my $buttons;						# Buttons used

	my $choice;						# Choice of the matter
	my $message;						# Display message


	#####################
	# Program area	
	
	# Addition operation
	if ( defined($param->{$config->{'TxtSubmitAddition'}}) ) 
	{
		# Retreive information
		($choice) = Webchat::Database::DataGet($Database, 'SELECT PuppetName FROM Puppet WHERE PuppetName=?', $param->{'PUPADD'});

		# Verification
		if ((length($param->{'PUPADD'}) > 2) && (!defined($choice)))
		{
			# Insertion upon acceptance
			$Database->do("INSERT INTO Puppet 
				(PuppetName, 
                                 PuppeteerLogin,
				 PuppetTextSize)
				VALUES(?,?,?)", {},
			 $param->{'PUPADD'}, $param->{'USER'}, 3);

			# Display status
			$message = $config->{'MsgCreate'};
		}
		else
		{ $message = $config->{'MsgExists'}; }
	}

	# Removal operation
	if ( defined($param->{$config->{'TxtSubmitDelete'}}) ) 
	{
		# Retreive information
		($choice) = Webchat::Database::DataGet($Database, 'SELECT PuppetName FROM Puppet WHERE PuppetName=? AND PuppeteerLogin=?', $param->{'PUPPET'}, $param->{'USER'});

		# Verification
		if (defined($choice))
		{
			# Removal of secondary tables
			$Database->do("DELETE FROM PuppetInvitation WHERE PuppetName=?", {}, $param->{'PUPPET'} ); 
			$Database->do("DELETE FROM PuppetIgnore WHERE PuppetName=?", {}, $param->{'PUPPET'} ); 

			# Removal of primary tables
			$Database->do("DELETE FROM Puppet WHERE PuppetName=?", {}, $param->{'PUPPET'} );

			# Message of success
			$message = $config->{'MsgDeletePass'}; 
		}		
		else
		{ $message = $config->{'MsgDeleteFail'}; }
	}


	# Cleanup
	delete $param->{$config->{'TxtSubmitAddition'}};
	delete $param->{$config->{'TxtSubmitDelete'}};


	# Display of values
	# Retreive a list of puppets
	Webchat::Database::GetListPuppetYours($Database, \@puppets, $param->{'USER'});

	# Prepare widgets
	$list       = $cgi->scrolling_list('PUPPET', \@puppets, $puppets[0], 10);
	$buttonview = $cgi->submit($config->{'TxtSubmitView'});
	$buttonrem  = $cgi->submit($config->{'TxtSubmitDelete'});
	$buttons    = "$buttonview<BR><BR>$buttonrem";

	# Selection/removal
	# Beginning of table and form
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);

	# Status message
	if (defined($message)) { Webchat::Table::MakeSingle($cgi, $message); Webchat::Table::MakeBlank($cgi); }
	
	# Select puppet
	Webchat::Table::MakeSingle($cgi, $config->{'TagPupDelSelect'});
	Webchat::Table::MakeValid($cgi, $list, $buttons);
	Webchat::Table::MakeBlank($cgi);

	# Select kit
	Webchat::Table::MakeSingle($cgi, $config->{'TagPupAddition'});
	Webchat::Table::MakeValid($cgi, $cgi->textfield('PUPADD', '', 20, 30), $cgi->submit($config->{'TxtSubmitAddition'}));
	Webchat::Table::MakeBottom($cgi);

	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form();

}


#####################
# PuppetControl
#
# Sub-routine that will handle the change of information for a puppet.  This
# is assuming that the submit button was pressed.

sub PuppetControl
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter handle
	my $config = shift;					# Config hash

	my $url = $cgi->url();					# Self reference
	my $inline;						# Parameters

	my @info;						# Gathered information


	#####################
	# Program area

	# Change of information and verify for appripriate button press
	if ( defined($param->{$config->{'TxtSubmitButton'}}) )
	{ PuppetControlWrite($Database, $param); }

	# Back button
	# Generate Inline
	$inline = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});	

	# Display
	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, "<A HREF=\"$url$inline\">$config->{TagBack}</A>");
	Webchat::Table::MakeBottom($cgi);


	# General display
	# Gather information
	@info = Webchat::Database::DataGetPuppetInfo($Database, $param->{'PUPPET'}); 

	# Add hint for users
	$info[3] = (defined($info[3])) ? $info[3] : 'http://';
	$info[7] = (defined($info[7])) ? $info[7] : 'http://';

	# Display of information
	# Write top of table and form
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);

	# Most clueful information
	Webchat::Table::MakeValid($cgi, $config->{'TagPupName'}, "<B>$info[0]</B>");
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Image information
	Webchat::Table::MakeValid($cgi, $config->{'TagPupImageLink'}, $cgi->textfield('PUPIMG', $info[3], 30, 200));
	Webchat::Table::MakeValid($cgi, $config->{'TagPupImgWidth'},  "<I>$info[5]</I>");
	Webchat::Table::MakeValid($cgi, $config->{'TagPupImgHeight'}, "<I>$info[4]</I>");
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Bio Url
	Webchat::Table::MakeValid($cgi, $config->{'TagPupBio'}, $cgi->textfield('PUPBIO', $info[7], 30, 200));
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# More complexe tagline
	Webchat::Table::MakeSingle($cgi, $config->{'TagPupTagline'});
	Webchat::Table::MakeSingle($cgi, "<TEXTAREA NAME=\"PUPTAG\", ROWS=3, COLS=65, WRAP=\"SOFT\">$info[2]</TEXTAREA>" );
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# More complexe title
	Webchat::Table::MakeSingle($cgi, $config->{'TagPupTitle'});
	Webchat::Table::MakeSingle($cgi, "<TEXTAREA NAME=\"PUPTIT\", ROWS=3, COLS=65, WRAP=\"SOFT\">$info[6]</TEXTAREA>");
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Finish table
	Webchat::Table::MakeValid($cgi, $cgi->submit($config->{'TxtSubmitButton'}), $cgi->reset($config->{'TxtResetButton'}));
	Webchat::Table::MakeBottom($cgi);

	# Embed 
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'PUPPET', $param->{'PUPPET'});

	# End table and form
	Webchat::Table::MakeBottom($cgi);
	print $cgi->end_form();
}


#####################
# PuppetControlWrite
#
# Sub-routine that will handle the change of information for a puppet.  This
# is used as a sub process to handle database interaction.

sub PuppetControlWrite
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $param = shift;					# Parameter handle

	my $height;						# Image height
	my $width;						# Image width

	my $ctit;						# Cleaned title
	my $ctag;						# Cleaned tag
	my $cbio;						# Bio


	#####################
	# Program area

	# Image handling
	# If valid URL
	if (length($param->{'PUPIMG'}) > 10) 
	{ 
		# Capture and determin size
		($width, $height, $_) = ImageSize($Database, $param->{'PUPIMG'}); 
	
		# Update database with new information
		$Database->do("UPDATE Puppet
			SET PuppetPic=?, 
			    PuppetPicWidth=?, 
			    PuppetPicHeight=?
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?", {}, 
		 $param->{'PUPIMG'}, $width, $height, $param->{'PUPPET'}, $param->{'USER'}); 
	}
	else
	{
		# Update database with nulls
		$Database->do("UPDATE Puppet 
			SET PuppetPic=NULL, 
		            PuppetPicWidth=NULL, 
			    PuppetPicHeight=NULL
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?", {}, 
		 $param->{'PUPPET'}, $param->{'USER'}); 
	}

	# Data handling
	# Truncate and clean output
	$ctag = substr($param->{'PUPTAG'}, 0, 500);
	$ctit = substr($param->{'PUPTIT'}, 0, 500);
	$cbio = $param->{'PUPBIO'};

	# Tag handling
	($ctag) = Webchat::Filter::SafeHTML($ctag);
	($ctit) = Webchat::Filter::SafeHTML($ctit);

	# Database update
	# Tagline, Title and Bio link
 	$ctag = (length($ctag) > 3)  ? $ctag : undef;
 	$ctit = (length($ctit) > 3)  ? $ctit : undef;	
 	$cbio = ((length($cbio) > 12) && ($cbio =~ /^http:\/\//is)) ? $cbio : undef; 

	# Update fields
	$Database->do("UPDATE Puppet 
		SET PuppetTag=?, 
		    PuppetTitle=?,
		    PuppetBio=? 
		WHERE PuppetName=? 
		  AND PuppeteerLogin=?", {}, 
	 $ctag, $ctit, $cbio, $param->{'PUPPET'}, $param->{'USER'});
}
