#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 03/10/2001
#################################################################################
#
# Config Realm - Script part of Webchat designed fro supervisors to control realms
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
#use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table handler


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my $inline; 							# Inline parameters
my $url;							# Self referencing link

my %config;							# Configuration hash
my %param;							# Parameter hash


#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle
	$url = $cgi->url();					# Retreive URL

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashConfiguration($Database, \%config);
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	if ( Webchat::Login::GetVerificationSuper($Database, $cgi, \%param) )
	{
		print "$config{'TagTitle'}\n<BR>\n";		# Display title

		# Realm selection
		unless (defined($param{'ROOM'}))
		{ RealmSelect($Database, $cgi, \%param, \%config); }

		# Realm handling
		else
		{ 
			# Back button
			# Generate inline
			($inline) = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param{'USER'}, 'CRYPT', $param{'CRYPT'}); 

			# Display URL
		 	print "<A HREF=\"$url$inline\">$config{TagBack}</A>";	


			# Room information handling and display basics
			RoomWrite($Database, \%param, \%config);
			RoomView($Database, $cgi, \%param, \%config);	

			# Bannings
			print "$config{'TagRealmBannings'}";
			RoomBan($Database, $cgi, \%param, \%config);	

			# Word Restrictions
			print "$config{'TagRealmWords'}";
			RoomWord($Database, $cgi, \%param, \%config);	
		}
	}

	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Audit Trail
#
# Only used for supervisors, as it was deemed that adminstrators could be trusted
# and also have the access to modify results as needed.  Supervisors on the other
# hand should not.

sub AuditTrail							# Database, Parameters
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $param = shift;					# Parameter hash
	my $action = shift;					# Action to take

	#####################
	# Program area

	# Insert into audit trail
	$Database->do("INSERT INTO Audit 
		(AuditNumber,
		 PuppeteerLogin,
		 AuditAction,
		 AuditTimestamp)
		VALUES (NULL, ?, ?, NOW())", {}, 
	 $param->{'USER'}, $action);
}

# Alias for bannings
# Add
sub AuditTrailBanAdd
{ AuditTrail(shift, shift, "$_[0]: Addition of puppeteer $_[1] to the banned list..."); }

# Remove
sub AuditTrailBanRem
{ AuditTrail(shift, shift, "$_[0]: Removal of puppeteer $_[1] from the banned list..."); }

# Alias for Words
# Add
sub AuditTrailWordAdd
{ AuditTrail(shift, shift, "$_[0]: Addition of word $_[1] to the word restriction list..."); }

# Remove
sub AuditTrailWordRem
{ AuditTrail(shift, shift, "$_[0]: Removal of word $_[1] from the word restriction list..."); }

# Alias for Information
# add
sub AuditTrailInfoChange
{ AuditTrail(shift, shift, "$_[0]: Realm information was changed..."); }

# Remove
sub AuditTrailInfoReset
{ AuditTrail(shift, shift, "$_[0]: Realm information was reset..."); }

# Alias for Realm
sub AuditTrailRealm
{ AuditTrail(shift, shift, "$_[0]: The realms primary information was changed/updated..."); }


#####################
# OptionSplit
#
# Splits options list into seperate elements of array

sub OptionSplit
{
	#####################
	# Data members

	my $array = shift;					# Reference to array
	my $data = shift;					# Retreived line


	#####################
	# Program area
	@{ $array } = split(/:/, $data);			# Split line
}


#####################
# Realm Select
#
# Basic display and prompt user for a selection on which realm to adminster and
# make modifications of some type to. 

sub RealmSelect							# Database, CGI, Parameter, config
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter hash
	my $config = shift;					# Config hash

	my @realms;						# Array of realms
	my $realms;						# Popup menu


	#####################
	# Program area

	# Prepare of drop down
	Webchat::Database::GetListPublicRealms($Database, \@realms);
	$realms = $cgi->popup_menu('ROOM', \@realms);

	# HTML Display
	# Star form and top
	print $cgi->start_form();				# Begin form
	Webchat::Table::MakeTop($cgi);

	# Relam selection
	Webchat::Table::MakeValid($cgi, $config->{'TagRealmRealmSelect'}, $realms);
	Webchat::Table::MakeBlank($cgi);

	# Submit and bottom
	Webchat::Table::MakeSingle($cgi, $cgi->submit($config->{'TxtSubmitView'}));
	Webchat::Table::MakeBottom($cgi);

	# End of form and embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});# Embed parameters
	print $cgi->end_form();					# End form	
}


#####################
# Room Ban
#
# Controls the bannings of certain individuals, this is realm specific.   As a result
# a complete ban can result in a password change that will cause the user to not be
# able to log on any furthur

sub RoomBan
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter hash
	my $config = shift;					# Config hash

	my $jailbirds;						# List of jailbird puppets
	my $suspects;						# Possible suspects
	my $bird;						# Singular bird

	my @banned;						# List of banned puppeteers			
	my @jailbirds;						# List of jailbird puppets
	my @suspects;						# Possible suspects


	#####################
	# program area

	# Retreive list of banned and suspects
	Webchat::Database::GetListBanned($Database, \@banned, $param->{'ROOM'});
	Webchat::Database::GetListPuppetOther($Database, \@suspects, 'root');

	# Retreive a list of all banned puppets
	foreach $bird (@banned)
	{ Webchat::Database::GetListPuppetYours($Database, \@jailbirds, $bird); }

	# Format lists
	$jailbirds = $cgi->scrolling_list('BANNED',  \@jailbirds, $jailbirds[0], 10);
	$suspects  = $cgi->scrolling_list('SUSPECT', \@suspects,  $suspects[0],  10);


	# HTML
	# Beginning of form
	print $cgi->start_form();

	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeValid($cgi, $jailbirds, $cgi->submit($config->{'TxtSubmitDelete'}));
	Webchat::Table::MakeValid($cgi, $suspects,  $cgi->submit($config->{'TxtSubmitAddition'}));
	Webchat::Table::MakeBottom($cgi);

	# End of form
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'BAN', 'true');
	print $cgi->end_form();
}


#####################
# RoomView
# 
# Will allow the supervisors to view the selected room and make changes as necessary
# and will probably make the changes themselves.

sub RoomView
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter hash
	my $config = shift;					# Config hash

	my $access;						# Access control
	my $language;						# Language restriction
	my $guests;						# Guests access
	my $tags;						# Tag enforcement

	my @access;						# Access control
	my @language;						# Language restriction
	my @guests;						# Guests access
	my @tags;						# Tag enforcement

	my @realm;						# Realm information


	#####################
	# Program area

	# Data gathering and preperation
	# Populate option arrays
	OptionSplit(\@language, $config->{'OptLanugage'});
	OptionSplit(\@access,   $config->{'OptAccess'});
	OptionSplit(\@guests,   $config->{'OptGuests'});
	OptionSplit(\@tags,     $config->{'OptTags'});

	# Populate realm information array
	@realm = Webchat::Database::DataGetRealmCompleteInfo($Database, $param->{'ROOM'});

	# Prepare radiobuttons
	$language = $cgi->radio_group('LANG',   \@language, $realm[1], 'true');
	$access   = $cgi->radio_group('ACCESS', \@access,   $realm[2], 'true');
	$guests   = $cgi->radio_group('GUEST',  \@guests,   $realm[3], 'true');
	$tags     = $cgi->radio_group('TAGS',   \@tags,     $realm[5], 'true');


	# HTML display
	# Star form and top
	print $cgi->start_form();				# Begin form
	Webchat::Table::MakeTop($cgi);

	Webchat::Table::MakeSingle($cgi, "<B>$param->{'ROOM'}</B>");
	Webchat::Table::MakeBlank($cgi);
	
	# Realm Language
	Webchat::Table::MakeValid($cgi, $config->{'TagRealmLanguage'}, $language);
	Webchat::Table::MakeBlank($cgi);

	# Access control
	Webchat::Table::MakeValid($cgi, $config->{'TagRealmAccess'}, $access);
	Webchat::Table::MakeBlank($cgi);

	# Guest access
	Webchat::Table::MakeValid($cgi, $config->{'TagRealmGuests'}, $guests);
	Webchat::Table::MakeBlank($cgi);

	# Realm tag control
	Webchat::Table::MakeValid($cgi, $config->{'TagRealmTags'}, $tags);
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeBlank($cgi);

	# Image restriction
	Webchat::Table::MakeValid($cgi, $config->{'TagPupImgHeight'}, $cgi->textfield('IMGHEIGHT', $realm[10], 3, 3));
	Webchat::Table::MakeValid($cgi, $config->{'TagPupImgWidth'},  $cgi->textfield('IMGWIDTH', $realm[11], 3, 3));
	Webchat::Table::MakeBlank($cgi);
	
	# Realm info
	Webchat::Table::MakeSingle($cgi, $config->{'TagRealmInfo'});
	Webchat::Table::MakeSingle($cgi, $cgi->textarea('INFO', $realm[6], 10, 70));
	Webchat::Table::MakeBlank($cgi);

	# Submit and bottom
	Webchat::Table::MakeValid($cgi, $cgi->submit($config->{'TxtSubmitButton'}), $cgi->submit($config->{'TxtResetButton'}));
	Webchat::Table::MakeBottom($cgi);

	# End of form and embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'MAIN', 'True');	
	print $cgi->end_form();					# End form	
}


#####################
# Room Word
#
# Controls word restrictions, as a result one can add or remove words for which
# such words will be replaced from posts by ****.  Simplistic but effective.

sub RoomWord
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter hash
	my $config = shift;					# Config hash

	my $words;						# List of words
	my @words;						# List of words	


	#####################
	# program area

	# Retreive list of banned and suspects
	Webchat::Database::GetListWords($Database, \@words, $param->{'ROOM'});
	$words = $cgi->scrolling_list('WORDREM',  \@words, $words[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();

	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeValid($cgi, $words, $cgi->submit($config->{'TxtSubmitDelete'}));
	Webchat::Table::MakeValid($cgi, $cgi->textfield('WORDADD', '', 10, 10), $cgi->submit($config->{'TxtSubmitAddition'}));
	Webchat::Table::MakeBottom($cgi);

	# End of form
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'WORDS', 'True');
	print $cgi->end_form(), "\n";
}


#####################
# Room Write
# 
# Will handle the writing of information gathered from the forms. 

sub RoomWrite
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $param = shift;					# Parameter hash
	my $config = shift;					# Config hash

	my $puppeteer;						# Puppeteers name


	#####################
	# Program area
	
	# Word restrictions
	if (defined($param->{'WORDS'}))
	{
		# Removal
		if (defined($param->{"$config->{'TxtSubmitDelete'}"}))
		{ 
			$Database->do("DELETE FROM WordRestriction 
				WHERE RealmName=? 
				  AND WordSpelling=?", {}, 
			 $param->{'ROOM'}, $param->{'WORDREM'}); 
			AuditTrailWordRem($Database, $param, $param->{'ROOM'}, $param->{'WORDREM'});
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{ 
			$Database->do("INSERT INTO WordRestriction 
				(WordSpelling,
				 RealmName)
				VALUES (?,?)", {}, 
			 $param->{'WORDADD'}, $param->{'ROOM'}); 
			AuditTrailWordAdd($Database, $param, $param->{'ROOM'}, $param->{'WORDADD'});
		}	


		# Cleanup
		delete $param->{'WORDS'};
		delete $param->{$config->{'TxtSubmitDelete'}};
		delete $param->{$config->{'TxtSubmitAddition'}};
	}

	# For banning handling
	# Removal of person
	elsif (defined($param->{'BAN'}))
	{
		if (defined($param->{"$config->{'TxtSubmitDelete'}"}))
		{
			# Retreive and remove puppeteer from banned list
			($puppeteer) = Webchat::Database::DataGetPuppeteerLogin($Database, $param->{'BANNED'});
			$Database->do("DELETE FROM PuppeteerRestriction WHERE PuppeteerLogin=?", {}, $puppeteer);
			AuditTrailBanRem($Database, $param, $param->{'ROOM'}, $puppeteer);
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{
			# Retreive and remove puppeteer from banned list
			($puppeteer) = Webchat::Database::DataGetPuppeteerLogin($Database, $param->{'SUSPECT'});

			$Database->do("INSERT INTO PuppeteerRestriction 
				(PuppeteerLogin,
				 RealmName,
				 RestrictionTimestamp)	
				VALUES (?,?,?)", {}, 
			 $puppeteer, $param->{'ROOM'}, time);
			AuditTrailBanAdd($Database, $param, $param->{'ROOM'}, $puppeteer);
		}

		# Cleanup
		delete $param->{'BAN'};
		delete $param->{$config->{'TxtSubmitDelete'}};
		delete $param->{$config->{'TxtSubmitAddition'}};
	}

	# Update main information
	elsif (defined($param->{'MAIN'}))
	{
		# Update primary information
		$Database->do("UPDATE Realm 
			SET RealmLanguage=?, 
			    RealmAccess=?, 
			    RealmGuests=?, 
			    RealmTags=?,
			    RealmImageHeight=?, 
			    RealmImageWidth=? 
			WHERE RealmName=?", {}, 
		 $param->{'LANG'}, $param->{'ACCESS'}, $param->{'GUEST'}, $param->{'TAGS'},
		 $param->{'IMGHEIGHT'}, $param->{'IMGWIDTH'}, $param->{'ROOM'});
		AuditTrailRealm($Database, $param, $param->{'ROOM'});
			

		# Determine Realm Information
		$param->{'INFO'} = (length($param->{'INFO'}) > 4) ? $param->{'INFO'} : undef;

		# Enter information
		$Database->do("UPDATE Realm 
			SET RealmInfo=? 
			WHERE RealmName=?", {}, 
		 $param->{'INFO'}, $param->{'ROOM'});
		AuditTrailInfoChange($Database, $param, $param->{'ROOM'});

		# Cleanup
		delete $param->{'MAIN'};
	}
}
