#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 02/22/2001
#################################################################################
#
# Registration - Script part of Webchat designed to register patrons 
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################
use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
#use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table module


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my $capture;							# Arbitrary captured line

my %login;							# Login hash
my %system;							# System hash
my %param;							# Parameter hash


#################################################################################
# Program Area
#################################################################################

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	$cgi = new CGI;						# Create CGI Handle

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML Header

	unless ($cgi->param())					# Entrance without parameters
	{
		InformationFormInit($Database, $cgi, \%param);	# Initial form
	}

	else							# Entrace with parameters
	{
		Webchat::Param::GetParam($cgi, \%param);	# Retreive parameters

		# If no registration codes or logins		
		unless ((defined($param{'LOGIN'})) && (defined($param{'CONFCODE'})))
		{
			# If not accepted show AUP
			unless (defined($param{'ACCEPT'}))
			{ 
				# Check contrains if correct then print AUP
				if (ConstraintHandler($Database, $cgi, \%param))
				{ PolicyForm($Database, $cgi, \%param); }
			}

			# If accepted process request
			elsif (defined($param{'ACCEPT'}))
			{
				RequestHandler($Database, $cgi, \%param);			
			}
		}

		# If such a code was sent
		else
		{ 
			# Check if registration is valid
			if (AcceptHandler($Database, \%param))
			{ Webchat::Database::DocumentGetAccepted($Database); }
			
			# If not, redisplay instructions
			else
			{ Webchat::Database::DocumentGetRegistration($Database); }

		}
	}

	
	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect from database


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Acceptence Handler
#
# Will determine is registration code is correct.  If so, the script will remove
# registration information allowing user to login as needed.

sub AcceptHandler						# Database, Param reference
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle

	my $param = shift;					# Parameter list
	my $capture;						# Captured data

	#####################
	# Program area
	
	# Prepare and execute statement
	$Statement = $Database->prepare("SELECT PuppeteerLogin 
		FROM PuppeteerRegistration
		WHERE PuppeteerLogin=? 
		  AND PuppeteerConfCode=?"); 
	$Statement->execute($param->{'LOGIN'}, $param->{'CONFCODE'});
	
	# Capture and finish statement
	$capture = $Statement->fetchrow();
	$Statement->finish;

	# If entry was found
	if (defined($capture))
	{
		# Erase lock
		$Database->do("DELETE 
			FROM PuppeteerRegistration 
			WHERE PuppeteerLogin=? 
			  AND PuppeteerConfCode=?", {}, 
		 $param->{'LOGIN'}, $param->{'CONFCODE'});

		# Return true
		return 1;
	}

	return 0;
}


#####################
# Check Parameter
# 
# Checks parameters and verifies if data exists already in table or if they
# fit criteria.

sub CheckParam
{
	#####################
	# Data members
	my $subject = shift; 					# Arbitrary retrieval
	my $choice  = shift;					# Choice on the matter
	my $check   = shift;					# Check value

	#####################
	# Program area

	# Lenght
	if ($choice == 0)
	{
		if (length($subject) < 3)
		{ return 0; }					# False if password short

		return 1;					# Return true if 5 or more
	}

	# E-Mail
	if ($choice == 1)
	{
		if ($subject !~ /\@/)
		{ return 0; }					# False if password short

		return 1;					# Return true if 5 or more
	}

	# Login name availability
	elsif ($choice == 2)
	{
		# Initialize and execute statement
		my $Statement = $subject->prepare("SELECT PuppeteerLogin 
			FROM  Puppeteer 
			WHERE PuppeteerLogin=?");
		$Statement->execute($check);

		# Retreive information and finish
		$choice = $Statement->fetchrow();
		$Statement->finish;
		
		# Verify fate
		if (defined($choice))				# Return false if not found
		{ return 0; }	

		return 1;					# Return true if not found	
	}
}

# Alias for Lenght
sub CheckParamLenght
{ return CheckParam(shift, 0); }

# Alias for Email
sub CheckParamEmail
{ return CheckParam(shift, 1); }

# Alias for Login
sub CheckParamLogin
{ return CheckParam(shift, 2, shift); }


#####################
# Constraint Handler
#
# Checks contraints, if they are correct returns true, otherwise requests
# repost of data missing or off.

sub ConstraintHandler						# Database, CGI, Parameter Reference
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter reference


	#####################
	# Program area

	# Username/login verfication
	unless ((CheckParamLogin($Database, $param->{'USERNAME'})) && (CheckParamLenght($param->{'USERNAME'})))		
	{ InformationFormLogin($Database, $cgi, $param); return 0;}	# Return false

	# Full name verification
	unless (CheckParamLenght($param->{'FULLNAME'}))
	{ InformationFormFullname($Database, $cgi, $param); return 0;}	# Return false

	# EMail check
	unless ((CheckParamLenght($param->{'EMAIL'})) && (CheckParamEmail($param->{'EMAIL'})))
	{ InformationFormEmail($Database, $cgi, $param);  return 0;}	# Return false

	# Password verification
	unless (($param->{'PASSONCE'} eq $param->{'PASSTWICE'}) && (CheckParamLenght($param->{'PASSONCE'})))
	{ InformationFormPassword($Database, $cgi, $param); return 0;}	# Return false

	return 1;
}


#####################
# Information Form
#
# Initially displayed sub-routine, for sole purpose of gathering infromation.

sub InformationForm
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter handling
	my $choice = shift;					# Choice of the matter

	my %login;						# Login hash
	my %system;						# System hash

	my @javascript;						# Javascript prefs array
	my @chatpref;						# Chat prefs array
	my @formatting;						# Formatting options

	my $javascript;						# Javascript option
	my $chatpref;						# Chatpref option
	my $formatting;						# Formatting option


	#####################
	# Program area

	# Link to hashes
	Webchat::Database::GetHashLogin($Database, \%login);
	Webchat::Database::GetHashSystem($Database, \%system);

	# Retreive arrays with necessary options
	OptionSplit(\@javascript, $login{'OptJavascript'});
	OptionSplit(\@chatpref, $login{'OptChatPref'});
	OptionSplit(\@formatting, $login{'OptFormat'});

	Webchat::Database::DocumentGetRegistration($Database);	# Information on registration


	print $cgi->start_form();				# Begin form

	Webchat::Table::MakeTop($cgi);
	# Main information
	# Username
	if ($choice == 99)					# Initial
	{ Webchat::Table::MakeValid($cgi, $login{'TagUsername'}, $cgi->textfield('USERNAME', '', 10, 10)); }
	elsif ($choice == 0)					# Invalid
	{ Webchat::Table::MakeInvalid($cgi, $login{'TagUsername'}, $cgi->textfield('USERNAME', '', 10, 10)); }
	else							# Valid repeat
	{ Webchat::Table::MakeValid($cgi, $login{'TagUsername'}, $cgi->textfield('USERNAME', $param->{'USERNAME'}, 10, 10)); }

	# Fullname
	if ($choice == 99)					# Initial
	{ Webchat::Table::MakeValid($cgi, $login{'TagFullName'}, $cgi->textfield('FULLNAME', '', 20, 30)); }
	elsif ($choice == 1)					# Invalid
	{ Webchat::Table::MakeInvalid($cgi, $login{'TagFullName'}, $cgi->textfield('FULLNAME', '', 20, 30)); }
	else							# Valid repeat
	{ Webchat::Table::MakeValid($cgi, $login{'TagFullName'}, $cgi->textfield('FULLNAME', $param->{'FULLNAME'}, 20, 30)); }

	# EMail
	if ($choice == 99)					# Initial
	{ Webchat::Table::MakeValid($cgi, $login{'TagEMail'}, $cgi->textfield('EMAIL', '', 20, 45)); }
	elsif ($choice == 2)					# Invalid
	{ Webchat::Table::MakeInvalid($cgi, $login{'TagEMail'}, $cgi->textfield('EMAIL', '', 20, 45)); }
	else
	{ Webchat::Table::MakeValid($cgi, $login{'TagEMail'}, $cgi->textfield('EMAIL', $param->{'EMAIL'}, 20, 45)); }

	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Password information
	if ($choice == 99)					# Initial
	{
		Webchat::Table::MakeValid($cgi, $login{'TagPassword'}, $cgi->password_field('PASSONCE', '', 10, 8));
		Webchat::Table::MakeValid($cgi, $login{'TagPasswordRepeat'}, $cgi->password_field('PASSTWICE', '', 10, 8));
	}
	elsif ($choice == 3)					# Invalid
	{
		Webchat::Table::MakeInvalid($cgi, $login{'TagPassword'}, $cgi->password_field('PASSONCE', '', 10, 8));
		Webchat::Table::MakeInvalid($cgi, $login{'TagPasswordRepeat'}, $cgi->password_field('PASSTWICE', '', 10, 8));
	}
	else							# Valid Repeat
	{
		Webchat::Table::MakeValid($cgi, $login{'TagPassword'}, $cgi->password_field('PASSONCE', $param->{'PASSONCE'}, 10, 8));
		Webchat::Table::MakeValid($cgi, $login{'TagPasswordRepeat'}, $cgi->password_field('PASSTWICE', $param->{'PASSTWICE'}, 10, 8));
	}
		

	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Initialize Javascript and Chat preferences
	$javascript = $cgi->radio_group('JAVASCRIPT', \@javascript, $javascript[0],'true');
	$chatpref = $cgi->radio_group('CHATPREFS', \@chatpref, $chatpref[0],'true');
	$formatting = $cgi->radio_group('FORMATPREFS', \@formatting, $formatting[0],'true');
	
	# Javascript options
	Webchat::Table::MakeValid($cgi, $login{'TagOptJavacript'}, $javascript);
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Chat preferences
	Webchat::Table::MakeValid($cgi, $login{'TagOptChatPref'}, $chatpref);
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Formatting preferences
	Webchat::Table::MakeValid($cgi, $login{'TagOptFormat'}, $formatting);
	Webchat::Table::MakeBlank($cgi);			# Spacer
	

	# Submission
	Webchat::Table::MakeValid($cgi, $cgi->submit($login{'TxtSubmitButtonSub'}), $cgi->reset($login{'TxtResetButton'}));
	Webchat::Table::MakeBottom($cgi);
	
	print $cgi->end_form();					# End form
}

# Alias for Initilization
sub InformationFormInit
{ InformationForm(shift, shift, shift, 99); }

# Alias for Login
sub InformationFormLogin
{ InformationForm(shift, shift, shift, 0); }

# Alias for Fullname
sub InformationFormFullname
{ InformationForm(shift, shift, shift, 1); }

# Alias for EMail
sub InformationFormEmail
{ InformationForm(shift, shift, shift, 2); }

# Alias for pasword
sub InformationFormPassword
{ InformationForm(shift, shift, shift, 3); }


#####################
# Policy Form
#
# Displays the acceptable use policy flag and embeds all information gathered.
# Also adds a tag to mark acceptance.

sub PolicyForm							# Database, CGI, Parameter hash
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# Common gateway interface
	my $param = shift;					# Parameter

	my $single;						# Single unit

	my %login;						# Login hash dbm


	#####################
	# Program area
	Webchat::Database::GetHashLogin($Database, \%login);	# Link with hash
	Webchat::Database::DocumentGetAUP($Database);		# Display Acceptable use Policy


	# Cleanup
	delete $param->{$login{'TxtSubmitButtonSub'}};


	# Form display
	print $cgi->start_form();				# Beginning of form

	# Embed information gathered
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'ACCEPT', 'True');

	# Print access button
	print "\t", $cgi->submit($login{'TxtSubmitButtonAccept'}), "\n";
	
	print $cgi->end_form();					# End of form
}


#####################
# Request Handler
#
# Takes in the parameters, sending a part of the information to the database, then
# sending a confirmation E-Mail to the poster.

sub RequestHandler						# Database, CGI, Parameter hash
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# Common Gateway Interface
	my $param = shift;					# Parameter hash

	my $invitation;						# Invitation	

	my $confcode;						# Confirmation code
	my $inline;						# Authenthication string

	my $url = $cgi->url();					# Home location

	my %system;						# System hash to DBM
	my %login;						# Login hash to DBM

	#####################
	# Program area

	# Link to hashes
	Webchat::Database::GetHashLogin($Database, \%login);
	Webchat::Database::GetHashSystem($Database, \%system);

	# Information on registration
	Webchat::Database::DocumentGetRegistration($Database);	
	$invitation = Webchat::Database::DocumentGetInvitation($Database);

	# Confirmation code
	$confcode = md5_hex(rand());
	$confcode = substr($confcode, 0, 10);


	# Create authenthication string
	$inline = Webchat::Param::EmbedInline($Database, $cgi, 'LOGIN', $param->{'USERNAME'}, 'CONFCODE', $confcode, 'ACCEPT', 'True');

	# Message information
	if (defined($invitation))
	{
		# Go though all keywords and replace with real value
		$invitation =~ s/AUTHSTRING/$url$inline/g;
		$invitation =~ s/FULLNAME/$param->{'FULLNAME'}/g;
		$invitation =~ s/LOGINNAME/$param->{'USERNAME'}/g;
		$invitation =~ s/PORTAL/$system{'LocScriptPortal'}/g;
		$invitation =~ s/RETURNEMAIL/$system{'SetInfoContactAddress'}/g;
		$invitation =~ s/SERVERNAME/$system{'SetInfoServer'}/g;
	}

	
	# Define and open sendmail
	open(SENDMAIL, "|$system{'LocSendmail'} -oi -t")
		or die("Cannot fork sendmail: $!");	

	# Send out invitation
	print SENDMAIL "From: $system{'SetInfoContactName'} <$system{'SetInfoContactAddress'}>\n";
	print SENDMAIL "To: $param->{'FULLNAME'} <$param->{'EMAIL'}>\n";
	print SENDMAIL "Subject: Welcome to $system{'SetInfoServer'}\n\n";
	print SENDMAIL "$invitation\n";
	close(SENDMAIL);

	# Insert Puppeteer
	$Database->do("INSERT INTO Puppeteer 
		(PuppeteerLogin,
		 PuppeteerPassword,
		 PuppeteerName,
		 PuppeteerEMail,
		 PuppeteerTimestamp,
		 PuppeteerFormatting,
		 PuppeteerChatPref,
		 PuppeteerJavascript,
		 PuppeteerAdminPrivs,
		 PuppeteerSuperPrivs)
		VALUES (?,?,?,?,?,?,?,?, 'no', 'no')", {}, 
	 $param->{'USERNAME'}, crypt($param->{'PASSONCE'}, $confcode), 
	 $param->{'FULLNAME'}, $param->{'EMAIL'}, time,
	 OptionFindFormatting($login{'OptFormat'},$param->{'FORMATPREFS'}),
	 OptionFindChatprefs($login{'OptChatPref'},$param->{'CHATPREFS'}),
	 OptionFindJavascript($login{'OptJavascript'},$param->{'JAVASCRIPT'}));

	$Database->do("INSERT INTO PuppeteerRegistration 
		(PuppeteerLogin,
		 PuppeteerConfCode,
		 PuppeteerTimeStamp)
		VALUES (?,?,?)", {}, 
	 $param->{'USERNAME'}, $confcode, time);
}


#####################
# OptionSplit
# 
# Splits options list into seperate elements of array

sub OptionSplit
{
	#####################
	# Data members

	my $array = shift;					# Reference to array
	my $data = shift;					# Retreived line


	#####################
	# Program area
	@{ $array } = split(/:/, $data);			# Split line 
}


#####################
# OptionFind
# 
# Splits options list into seperate elements of array and determines which
# option was specifically selected for the database

sub OptionFind
{
	#####################
	# Data members
	my $string = shift;					# String with all options
	my $val = shift;					# Value to search for
	my $choice = shift;					# Flow of events

	my $counter = 0;					# Basic counter

	my @values;						# Array of values


	#####################
	# Program area

	OptionSplit(\@values, $string);				# Split options\receive values

	# Loop though array
	while ($string = shift @values)
	{
		# Verify for match
		if ($string eq $val)
		{ last; }					# If found exit loop
		
		$counter++;					# Increment if no match found
	}	

	# Javascript Options/Yes no Options
	if ($choice == 0)
	{
		# Which option applies
		if ($counter == 0) { return 'yes'; }
		elsif ($counter == 1) { return 'no'; }
		elsif ($counter == 2) { return 'yes'; }
	}

	# Chat preferences
	elsif ($choice == 1)
	{
		# Which option applies
		if ($counter == 0) { return 'basic'; }
		elsif ($counter == 1) { return 'framed'; }
		elsif ($counter == 2) { return 'stream'; }
		elsif ($counter == 3) { return 'basic'; }
	}

	# Format used in chat
	elsif ($choice == 2)
	{
		# Which option applies
		if ($counter == 0) { return 'complete'; }
		elsif ($counter == 1) { return 'imageless'; }
		elsif ($counter == 2) { return 'spartan'; }
		elsif ($counter == 3) { return 'complete'; }
	}
}


# Alias for Javascript
sub OptionFindJavascript
{ return OptionFind(shift, shift, 0); }

# Alias for Chat preferences
sub OptionFindChatprefs
{ return OptionFind(shift, shift, 1); }

# Alias for Formatting
sub OptionFindFormatting
{ return OptionFind(shift, shift, 2); }
